@@ -49,11 +49,11 @@ from handlers.neutron_responses import SUBNETS
 
 from handlers.selecting_handler import SelectingHandler
 
-from ovndb.ovn_north import OvnNorth
-from ovndb.ovn_north_mappers import NetworkMapper
-from ovndb.ovn_north_mappers import PortMapper
-from ovndb.ovn_north_mappers import Router
-from ovndb.ovn_north_mappers import RouterMapper
+from neutron.neutron_api_mappers import NetworkMapper
+from neutron.neutron_api_mappers import PortMapper
+from neutron.neutron_api_mappers import Router
+from neutron.neutron_api_mappers import RouterMapper
+from neutron.neutron_api import NeutronApi
 
 from ovntestlib import OvnRouterRow
 
@@ -315,9 +315,9 @@ class TestNeutronResponse(object):
 
     @mock.patch('ovsdbapp.backend.ovs_idl.connection', autospec=False)
     def test_post_routers(self, mock_connection):
-        nb_db = OvnNorth()
+        nb_db = NeutronApi()
         nb_db._add_router = Mock()
-        nb_db.atomics.idl = Mock()
+        nb_db.ovn_north.idl = Mock()
 
         nb_db._add_router.return_value = Router(OvnRouterRow(
             'uuid',
