@@ -22,32 +22,43 @@ import mock
 import pytest
 
 from ovsdbapp.backend.ovs_idl.idlutils import RowNotFound
-import ovndb.constants as ovnconst
-
+import constants as ovnconst
 from handlers.base_handler import ConflictError
+import neutron.constants as neutron_constants
+from neutron.neutron_api_mappers import InvalidRestData
+from neutron.neutron_api_mappers import MandatoryDataMissing
+from neutron.neutron_api_mappers import Network
+from neutron.neutron_api_mappers import NetworkMapper
+from neutron.neutron_api_mappers import NetworkPort
+from neutron.neutron_api_mappers import PortMapper
+from neutron.neutron_api_mappers import SecurityGroup
+from neutron.neutron_api_mappers import SecurityGroupMapper
+from neutron.neutron_api_mappers import SecurityGroupRuleMapper
+from neutron.neutron_api_mappers import SubnetConfigError
+from neutron.neutron_api_mappers import SubnetMapper
+from neutron.neutron_api_mappers import UnsupportedDataValueError
+from neutron.neutron_api import NeutronApi
 from ovirt_provider_config_common import dhcp_lease_time
 from ovirt_provider_config_common import dhcp_mtu
 from ovirt_provider_config_common import dhcp_server_mac
 from ovirt_provider_config_common import tenant_id
-from ovndb.ovn_north import OvnNorth
-from ovndb.ovn_north_mappers import Network
-from ovndb.ovn_north_mappers import NetworkMapper
-from ovndb.ovn_north_mappers import NetworkPort
-from ovndb.ovn_north_mappers import PortMapper
-from ovndb.ovn_north_mappers import SubnetConfigError
-from ovndb.ovn_north_mappers import SubnetMapper
-from ovndb.ovn_north_mappers import UnsupportedDataValueError
 
 from ovntestlib import assert_network_equal
 from ovntestlib import assert_port_equal
+from ovntestlib import assert_security_group_equal
+from ovntestlib import assert_security_group_rule_equal
 from ovntestlib import assert_subnet_equal
 from ovntestlib import NetworkApiInputMaker
 from ovntestlib import PortApiInputMaker
+from ovntestlib import SecurityGroupApiInputMaker
+from ovntestlib import SecurityGroupRuleApiInputMaker
 from ovntestlib import SubnetApiInputMaker
 from ovntestlib import OvnNetworkRow
 from ovntestlib import OvnPortRow
 from ovntestlib import OvnRouterPort
 from ovntestlib import OvnRouterRow
+from ovntestlib import OvnSecurityGroupRow
+from ovntestlib import OvnSecurityGroupRuleRow
 from ovntestlib import OvnSubnetRow
 
 
@@ -159,6 +170,38 @@ class TestOvnNorth(object):
 
     ROUTER_20 = OvnRouterRow(ROUTER_ID20, ROUTER_NAME20)
 
+    SECURITY_GROUP_ID = UUID(int=666)
+    SECURITY_GROUP_NAME = 'ultra-mega-security'
+    SECURITY_GROUP_DESCRIPTION = 'as safe as it gets'
+    SECURITY_GROUP = OvnSecurityGroupRow(
+        SECURITY_GROUP_ID,
+        SECURITY_GROUP_NAME,
+        external_ids={
+            SecurityGroupMapper.OVN_SECURITY_GROUP_DESCRIPTION:
+                SECURITY_GROUP_DESCRIPTION,
+            SecurityGroupMapper.OVN_SECURITY_GROUP_CREATE_TS: '',
+            SecurityGroupMapper.OVN_SECURITY_GROUP_UPDATE_TS: '',
+            SecurityGroupMapper.OVN_SECURITY_GROUP_REV_NUMBER: '1',
+            SecurityGroupMapper.OVN_SECURITY_GROUP_TENANT: tenant_id(),
+            SecurityGroupMapper.OVN_SECURITY_GROUP_PROJECT: tenant_id()
+        }
+    )
+
+    SECURITY_GROUP_RULE_ID_01 = UUID(int=1)
+    SECURITY_GROUP_RULE_01_EXT_IDS = {
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_SEC_GROUP_ID:
+            SECURITY_GROUP_ID,
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_PROTOCOL:
+            neutron_constants.PROTO_NAME_TCP,
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_ETHERTYPE:
+            neutron_constants.IPV4_ETHERTYPE
+    }
+    SECURITY_GROUP_RULE_01 = OvnSecurityGroupRuleRow(
+        SECURITY_GROUP_RULE_ID_01,
+        'from-lport', 'ip4 && tcp', 1001, SECURITY_GROUP_ID,
+        SECURITY_GROUP_RULE_01_EXT_IDS
+    )
+
     ports = [PORT_1, PORT_2]
 
     networks = [NETWORK_10, NETWORK_11, NETWORK_LOCALNET_12]
@@ -172,7 +215,7 @@ class TestOvnNorth(object):
     def test_get_networks(self, mock_ls_list, mock_connection):
         mock_ls_list.return_value.execute.return_value = TestOvnNorth.networks
 
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         result = ovn_north.list_networks()
 
         assert len(result) == 3
@@ -200,7 +243,7 @@ class TestOvnNorth(object):
         mock_ls_get.return_value.execute.return_value = (
             TestOvnNorth.NETWORK_10
         )
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         result = ovn_north.get_network(str(TestOvnNorth.NETWORK_ID10))
 
         network = Network(ls=TestOvnNorth.NETWORK_10, localnet_lsp=None)
@@ -219,7 +262,7 @@ class TestOvnNorth(object):
         mock_add_command.return_value.execute.return_value = (
             TestOvnNorth.NETWORK_10
         )
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = NetworkApiInputMaker(TestOvnNorth.NETWORK_NAME10).get()
         result = ovn_north.add_network(rest_data)
 
@@ -233,7 +276,8 @@ class TestOvnNorth(object):
             external_ids={
                 NetworkMapper.OVN_NETWORK_NAME: (
                     TestOvnNorth.NETWORK_NAME10
-                )
+                ),
+                NetworkMapper.OVN_NETWORK_PORT_SECURITY: 'False'
             }
         )
 
@@ -248,7 +292,7 @@ class TestOvnNorth(object):
             TestOvnNorth.NETWORK_LOCALNET_12)
         mock_ls_add_command.return_value.execute.return_value = (
             TestOvnNorth.NETWORK_LOCALNET_12)
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = NetworkApiInputMaker(
             TestOvnNorth.NETWORK_NAME12,
             provider_type=NetworkMapper.NETWORK_TYPE_VLAN,
@@ -272,7 +316,8 @@ class TestOvnNorth(object):
             external_ids={
                 NetworkMapper.OVN_NETWORK_NAME: (
                     TestOvnNorth.NETWORK_NAME12
-                )
+                ),
+                NetworkMapper.OVN_NETWORK_PORT_SECURITY: 'False'
             }
         )
         assert mock_lsp_add_command.call_count == 1
@@ -324,7 +369,7 @@ class TestOvnNorth(object):
             mock_dbset_command,
             mock_connection
     ):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         network_rest_data = NetworkApiInputMaker(
             TestOvnNorth.NETWORK_NAMEMTU, mtu=TestOvnNorth.VALUE_NETWORK_MTU
         ).get()
@@ -400,7 +445,7 @@ class TestOvnNorth(object):
     ):
         mock_dhcp_list_command.return_value.execute.return_value = []
 
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         network_rest_data = NetworkApiInputMaker(
             TestOvnNorth.NETWORK_NAMEMTU, mtu=TestOvnNorth.VALUE_NETWORK_MTU
         ).get()
@@ -467,7 +512,7 @@ class TestOvnNorth(object):
         autospec=False
     )
     def test_update_network(self, mock_set_command, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = NetworkApiInputMaker(TestOvnNorth.NETWORK_NAME10).get()
         result = ovn_north.update_network(rest_data, TestOvnNorth.NETWORK_ID10)
 
@@ -493,7 +538,7 @@ class TestOvnNorth(object):
         mock_ls_get_command.return_value.execute.return_value = (
             TestOvnNorth.NETWORK_LOCALNET_12
         )
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = NetworkApiInputMaker(
             TestOvnNorth.NETWORK_NAME12,
             provider_type=NetworkMapper.NETWORK_TYPE_VLAN,
@@ -535,7 +580,7 @@ class TestOvnNorth(object):
         lambda cmd, check_error: []
     )
     def test_delete_network(self, mock_del_command, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         ovn_north.delete_network(TestOvnNorth.NETWORK_ID10)
         assert mock_del_command.call_count == 1
         expected_del_call = mock.call(
@@ -575,14 +620,14 @@ class TestOvnNorth(object):
         autospec=False
     )
     @mock.patch(
-        'ovndb.ovn_north.ovs_version_29',
+        'neutron.neutron_api.ovs_version_29',
         lambda: True
     )
     def test_add_port(self, mock_db_set, mock_add_command, mock_connection):
         mock_add_command.return_value.execute.return_value = (
             TestOvnNorth.PORT_1
         )
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = PortApiInputMaker(
             TestOvnNorth.PORT_NAME01, str(TestOvnNorth.NETWORK_ID10),
             device_id=TestOvnNorth.DEVICE_ID,
@@ -658,6 +703,7 @@ class TestOvnNorth(object):
             ovnconst.TABLE_LSP,
             TestOvnNorth.PORT_ID01,
             (ovnconst.ROW_LSP_DHCPV4_OPTIONS, TestOvnNorth.SUBNET_ID101),
+            (ovnconst.ROW_LSP_PORT_SECURITY, []),
             (
                 ovnconst.ROW_LSP_ADDRESSES,
                 ['{mac_address} {ip_address}'.format(
@@ -681,7 +727,7 @@ class TestOvnNorth(object):
         lambda cmd, check_error: TestOvnNorth.ports
     )
     def test_list_ports(self, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         ports = ovn_north.list_ports()
         assert len(ports) == 2
         logical_switch = OvnNetworkRow(
@@ -718,7 +764,7 @@ class TestOvnNorth(object):
         autospec=False
     )
     def test_delete_port(self, mock_del_command, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         ovn_north.delete_port(TestOvnNorth.PORT_ID01)
         assert mock_del_command.call_count == 1
         expected_del_call = mock.call(
@@ -743,7 +789,7 @@ class TestOvnNorth(object):
         autospec=False
     )
     def test_delete_router_port(self, mock_del_command, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         with pytest.raises(ConflictError):
             ovn_north.delete_port(TestOvnNorth.PORT_ID01)
         assert mock_del_command.call_count == 0
@@ -754,7 +800,7 @@ class TestOvnNorth(object):
         lambda cmd, check_error: TestOvnNorth.subnets
     )
     def test_list_subnets(self, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         result = ovn_north.list_subnets()
         assert len(result) == 2
         assert result[0]['id'] == str(TestOvnNorth.SUBNET_ID101)
@@ -767,7 +813,7 @@ class TestOvnNorth(object):
         lambda cmd, check_error: TestOvnNorth.SUBNET_101
     )
     def test_get_subnet(self, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         result = ovn_north.get_subnet(TestOvnNorth.SUBNET_ID101)
         assert result['id'] == str(TestOvnNorth.SUBNET_ID101)
         assert result['network_id'] == str(TestOvnNorth.NETWORK_ID10)
@@ -788,7 +834,7 @@ class TestOvnNorth(object):
         lambda cmd, check_error: TestOvnNorth.NETWORK_10
     )
     def test_delete_subnet(self, mock_del_command, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         ovn_north.delete_subnet(TestOvnNorth.SUBNET_ID101)
         assert mock_del_command.call_count == 1
         expected_del_call = mock.call(
@@ -827,7 +873,7 @@ class TestOvnNorth(object):
                         mock_dbset_command, mock_connection):
         add_execute = mock_add_command.return_value.execute
         add_execute.return_value = TestOvnNorth.SUBNET_102
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = SubnetApiInputMaker(
             TestOvnNorth.SUBNET_102.external_ids.get(
                 SubnetMapper.OVN_NAME
@@ -905,7 +951,7 @@ class TestOvnNorth(object):
                                mock_dbset_command, mock_connection):
         add_execute = mock_add_command.return_value.execute
         add_execute.return_value = TestOvnNorth.SUBNET_102
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = SubnetApiInputMaker(
             'subnet_name', cidr=TestOvnNorth.SUBNET_CIDR,
             network_id=str(TestOvnNorth.NETWORK_ID10), dns_nameservers=[],
@@ -1022,7 +1068,7 @@ class TestOvnNorth(object):
         lambda cmd, check_error: TestOvnNorth.subnets
     )
     def test_subnet_add_duplicate_network(self, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = SubnetApiInputMaker(
             'subnet_name', cidr=TestOvnNorth.SUBNET_CIDR,
             network_id=str(TestOvnNorth.NETWORK_ID10), gateway_ip='1.1.1.0'
@@ -1031,7 +1077,7 @@ class TestOvnNorth(object):
             ovn_north.add_subnet(rest_data)
 
     def test_subnet_dhcp_enabled_false(self, mock_connection):
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = SubnetApiInputMaker(
             'subnet_name', cidr=TestOvnNorth.SUBNET_CIDR, network_id='',
             dns_nameservers=['1.1.1.1'], gateway_ip='1.1.1.0',
@@ -1047,7 +1093,7 @@ class TestOvnNorth(object):
         self, mock_ls_get_cmd, mock_connection
     ):
         mock_ls_get_cmd.side_effect = RowNotFound()
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         rest_data = SubnetApiInputMaker(
             'subnet_name', cidr=TestOvnNorth.SUBNET_CIDR, network_id=7,
             dns_nameservers=['1.1.1.1'], gateway_ip='1.1.1.0'
@@ -1102,7 +1148,7 @@ class TestOvnNorth(object):
             OvnNetworkRow(TestOvnNorth.NETWORK_ID11, ports=[port_row])
         ]
 
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         port = ovn_north.get_port(TestOvnNorth.PORT_ID01)
         assert port[PortMapper.REST_PORT_ADMIN_STATE_UP] == result
 
@@ -1111,7 +1157,7 @@ class TestOvnNorth(object):
     )
     def test_get_router(self, mock_lookup, mock_connection):
         mock_lookup.return_value = TestOvnNorth.ROUTER_20
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         result = ovn_north.get_router(str(TestOvnNorth.ROUTER_ID20))
 
         assert result['id'] == str(TestOvnNorth.ROUTER_ID20)
@@ -1131,7 +1177,7 @@ class TestOvnNorth(object):
     def test_delete_router(self, mock_lookup, mock_del_command,
                            mock_connection):
         mock_lookup.return_value = TestOvnNorth.ROUTER_20
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
 
         ovn_north.delete_router(str(TestOvnNorth.ROUTER_ID20))
 
@@ -1160,7 +1206,7 @@ class TestOvnNorth(object):
             TestOvnNorth.ROUTER_ID20,
             ports=[OvnRouterPort()]
         )
-        ovn_north = OvnNorth()
+        ovn_north = NeutronApi()
         with pytest.raises(ConflictError):
             ovn_north.delete_router(str(TestOvnNorth.ROUTER_ID20))
 
@@ -1168,3 +1214,212 @@ class TestOvnNorth(object):
             ovnconst.TABLE_LR, str(TestOvnNorth.ROUTER_ID20)
         )
         assert mock_del_command.call_count == 0
+
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbListCommand.execute',
+        lambda command, check_error: []
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda idl, table, uuid: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_get_security_group(self, mock_connection):
+        ovn_north = NeutronApi()
+        result = ovn_north.get_security_group(
+            TestOvnNorth.SECURITY_GROUP_ID
+        )
+        assert_security_group_equal(
+            result, SecurityGroup(
+                sec_group=TestOvnNorth.SECURITY_GROUP, sec_group_rules=[]
+            )
+        )
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_add_security_group(self, mock_connection):
+        ovn_north = NeutronApi()
+        rest_data = SecurityGroupApiInputMaker(
+            TestOvnNorth.SECURITY_GROUP_NAME, tenant_id(), tenant_id(),
+            description=TestOvnNorth.SECURITY_GROUP_DESCRIPTION
+        ).get()
+
+        result = ovn_north.add_security_group(rest_data)
+        security_group = SecurityGroup(
+            sec_group=TestOvnNorth.SECURITY_GROUP,
+            sec_group_rules=[]
+        )
+        assert_security_group_equal(result, security_group)
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_add_security_group_optional_data(self, mock_connection):
+        ovn_north = NeutronApi()
+        rest_data = SecurityGroupApiInputMaker(
+            TestOvnNorth.SECURITY_GROUP_NAME
+        ).get()
+
+        result = ovn_north.add_security_group(rest_data)
+        security_group = SecurityGroup(
+            sec_group=TestOvnNorth.SECURITY_GROUP,
+            sec_group_rules=[]
+        )
+        assert_security_group_equal(result, security_group)
+
+    def test_add_security_group_validator(self, mock_connection):
+        ovn_north = NeutronApi()
+        rest_data = SecurityGroupApiInputMaker(
+            TestOvnNorth.SECURITY_GROUP_NAME, tenant_id(), tenant_id(),
+            description=TestOvnNorth.SECURITY_GROUP_DESCRIPTION
+        ).get()
+
+        with pytest.raises(InvalidRestData) as invalid_data:
+            rest_data.update({'peanut-butter': 32})
+            ovn_north.add_security_group(rest_data)
+        assert (
+                'Invalid data found: peanut-butter'
+                == invalid_data.value.message
+        )
+
+    def test_add_security_group_validator_data_missing(self, mock_connection):
+        ovn_north = NeutronApi()
+        rest_data = SecurityGroupApiInputMaker(
+            TestOvnNorth.SECURITY_GROUP_NAME, tenant_id(), tenant_id(),
+            description=TestOvnNorth.SECURITY_GROUP_DESCRIPTION
+        ).get()
+
+        with pytest.raises(MandatoryDataMissing) as invalid_data:
+            rest_data.pop('name')
+            ovn_north.add_security_group(rest_data)
+        assert (
+                'Mandatory data name is missing'
+                == invalid_data.value.message
+        )
+
+    @mock.patch('ovsdbapp.schema.ovn_northbound.commands.PgDelCommand',
+                autospec=False)
+    def test_delete_security_group(self, mock_pg_del_command, mock_connection):
+        ovn_north = NeutronApi()
+
+        ovn_north.delete_security_group(TestOvnNorth.SECURITY_GROUP_ID)
+        assert mock_pg_del_command.call_count == 1
+        expected_del_call = mock.call(
+            ovn_north.idl, TestOvnNorth.SECURITY_GROUP_ID, if_exists=False
+        )
+        assert mock_pg_del_command.mock_calls[0] == expected_del_call
+
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbSetCommand',
+        autospec=False
+    )
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbListCommand.execute',
+        lambda command, check_error: []
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda self, table, the_id: TestOvnNorth.SECURITY_GROUP
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_update_security_group(self, mock_db_update, mock_connection):
+        ovn_north = NeutronApi()
+        rest_data = SecurityGroupApiInputMaker(
+            TestOvnNorth.SECURITY_GROUP_NAME, tenant_id(), tenant_id(),
+            description=TestOvnNorth.SECURITY_GROUP_DESCRIPTION
+        ).get()
+
+        ovn_north.add_security_group(rest_data)
+
+        # update the security group with other stuff
+        new_name = 'new_way_cuter_name'
+        new_description = 'this thing is waaaay better now'
+
+        ovn_north.update_security_group(
+            SecurityGroupApiInputMaker(
+                new_name, description=new_description
+            ).get(),
+            TestOvnNorth.SECURITY_GROUP_ID
+        )
+
+        assert mock_db_update.call_count == 1
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda idl, table, uuid: TestOvnNorth.SECURITY_GROUP_RULE_01
+    )
+    def test_get_security_group_rule(
+            self, mock_connection
+    ):
+        ovn_north = NeutronApi()
+        result = ovn_north.get_security_group_rule(
+            TestOvnNorth.SECURITY_GROUP_RULE_ID_01
+        )
+        assert_security_group_rule_equal(
+            result, TestOvnNorth.SECURITY_GROUP_RULE_01
+        )
+
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbListCommand.execute',
+        lambda command, check_error: [TestOvnNorth.SECURITY_GROUP_RULE_01]
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda idl, table, uuid: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_get_security_group_with_rules(self, mock_connection):
+        ovn_north = NeutronApi()
+        result = ovn_north.get_security_group(
+            TestOvnNorth.SECURITY_GROUP_ID
+        )
+        assert_security_group_equal(
+            result, SecurityGroup(
+                sec_group=TestOvnNorth.SECURITY_GROUP,
+                sec_group_rules=[TestOvnNorth.SECURITY_GROUP_RULE_01]
+            )
+        )
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAclAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP_RULE_01
+    )
+    def test_add_security_group_rules(self, mock_connection):
+        ovn_north = NeutronApi()
+        rest_data = SecurityGroupRuleApiInputMaker(
+            'ingress', TestOvnNorth.SECURITY_GROUP_ID, ether_type='IPv4',
+            protocol='tcp'
+        ).get()
+
+        result = ovn_north.add_security_group_rule(rest_data)
+        assert_security_group_rule_equal(
+            result, TestOvnNorth.SECURITY_GROUP_RULE_01
+        )
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup'
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAclDelCommand',
+        autospec=False
+    )
+    def test_delete_security_group_rule(
+            self, mock_delete_rule, mock_ovn_getter, mock_connection
+    ):
+        mock_ovn_getter.side_effect = [
+            TestOvnNorth.SECURITY_GROUP_RULE_01, TestOvnNorth.SECURITY_GROUP
+        ]
+        ovn_north = NeutronApi()
+        ovn_north.delete_security_group_rule(
+            TestOvnNorth.SECURITY_GROUP_RULE_ID_01
+        )
+        assert mock_delete_rule.call_count == 1
+        expected_del_call = mock.call(
+            ovn_north.idl, TestOvnNorth.SECURITY_GROUP_ID, 'from-lport', 1001,
+            'ip4 && tcp'
+        )
+        assert mock_delete_rule.mock_calls[0] == expected_del_call
